#!/usr/bin/perl
#GiBTool takes a list of Distinguished Names and creates vmr-like
#transactions and transmits them to a list of hosts via gsiftp. When a
#transfer fails it is queued up and at the next invocation of the
#script it attempts to send the failed transactions again to the 
#hosts which it failed on the first time around. If you want to get rid
#of the queued transactions edit them from $basedir/q/queueHosts.
#
# $Id: gibtool.pl,v 1.13 2001/11/01 18:43:20 srour Exp $
#
# Usage: gibtool.pl <input>
# Where <input> is a file with Distinguished Names separated by newlines.
# or
# Without any arguments it will just try and send any queued transactions.


#Set these variables.
$basedir = "/tmp/gibtool"; #base directory for created transactions
			   #and queued files.
#Change to gsiftpput if you want, uncomment the calls appropriate
#to gsiftpput in sendFiles() and sendQueued() and comment out the calls
#that use scp syntax.
$gsi = "scp"; #program to transfer files. Works best w/ key authentication.
$dir = "/home/srour"; #dir to put sent files on remote machine.

#Hosts to send files to.
@hosts = ("tux01","ferdinand.cs.wisc.edu","biron");


$totalTransactions = 0;
$numErrors = 0;
$numSuccess = 0;
$numIgnored = 0;
%HostQueues = ();
%Pairs = ();
$line = 0;

if(! -d "$basedir"){ mkdir("$basedir",0700) };
if(! -d "$basedir/q"){ mkdir("$basedir/q",0700) };
if(! -f "$basedir/q/queueHosts"){ `touch $basedir/q/queueHosts`; }
else{
    unless (open(QUEUE,"$basedir/q/queueHosts")){
	warn "Couldnt open $basedir/q/queueHosts: $!\n";
    }
    while(<QUEUE>){
	chomp;
	if($line eq "0"){
	    $count = $_;
	    if(!defined($count)){
		$count = 0;
	    }
	    $line++;
	    next;
	}

	@qfiles = split(":");
	$qhost = shift @qfiles;
	foreach $qfile (@qfiles){
	    push(@{ $HostQueues{$qhost} }, $qfile);
	}
	$line++;
    }
}
#foreach $host (keys %HostQueues){
#    print "$host\n";
#    foreach $file (@{$HostQueues{$host}}){
#	print "elem:$file\n";
#    }
#    print "\n";
#}


print "***** GiBTool Transaction Generator *****\n";

if(!defined($ARGV[0])){ 
    
    if(scalar(keys %HostQueues) > 0){
	sendQueued();
    }
    else{
    	print "Usage: $0 <input>\n";
	print "No queued files to send.\n";
    }
}
else{
    
    sendQueued();

    unless(open(FILE, "$ARGV[0]")){
	warn "Unable to open $ARGV[0]: $!\n";
	printStats();
	exit 1;
    }
	while(<FILE>) {
	    %Pairs = ();
	    $line = $_;
	    loadDN($line);
	    writeTransaction(%Pairs);
	    $count++;
	}
    close(FILE);

    sendFiles();
    
}

#Write a datafile with the hosts and files that could not be sent.
unless(open(QUEUE,">$basedir/q/queueHosts")){
    warn "Couldnt create/update queue file, $basedir/q/queueHosts: $!\n";
    printStats();
    exit 1;
}
print QUEUE "$count\n";
foreach my $group (keys %HostQueues) {
    my $size = @{HostQueues{$group}};
    if($size > 0){
	print QUEUE "$group";
	foreach (@{$HostQueues{$group}}) {
	    if(! -f $_){
		@parts=split(/\//,$_);
		$numparts=@parts;
		$filename=$parts[$numparts-1];
		print "Queued transaction $filename was sent.\n"; 
		next; 
	    }
	    print QUEUE ":$_";
	}
	print QUEUE "\n";
    }
	     }
close (QUEUE);

printStats();

exit 0;

#Load an individual DN and put the values into %Pair 
sub loadDN{
    $dn = $_;
    chomp($dn);
    $Pairs{"dn"} = $dn;
    @list = split("\/", $dn);
    foreach (@list) {
	($key, $value) = split("=",$_);
	if(!defined($key)){ next; }
	$Pairs{$key} = $value;
    }
}

#Create the transaction and place it in $basedir.
sub writeTransaction{
    $TID = GenTID();
    $date=`date +%m/%d/%Y_%T`;
    chomp($date);
    open(OUT, ">/tmp/gibtool/$TID.REQUEST_ACCOUNT_CREATE");
    print "Creating $TID with dn=$Pairs{'dn'}\n";
    print OUT "*transaction LES REQUEST_ACCOUNT_CREATE $TID $date 0\n";
    print OUT "dn:$Pairs{'dn'}\n";
    print OUT "organization:$Pairs{'O'}\n";
    print OUT "citizenship:$Pairs{'C'}\n";
    print OUT "dept:$Pairs{'OU'}\n";
    print OUT "alliance_psn:\n";
    print OUT "mach_num:\n";
    print OUT "ssn:\n";
    print OUT "title:\n";
    print OUT "email:\n";

    ($first,$middle,$last) = split(" ",$Pairs{'CN'});
    print OUT "first_name:$first\n";
    if(!(defined($last))){
	print OUT "middle_name:\n";
	print OUT "last_name:$middle\n";
    }
    else{
	print OUT "middle_name:$middle\n";
	print OUT "last_name:$last\n";
    }
    print OUT "*end";
    close (OUT); 
}

#Generate sequence number for transaction.
sub GenTID{
    my($myID,$newfirst,$temp);
    $temp = $count;
    if($temp > 999) { 
	$temp = 1; 
	$count = 1;
    }
    
    $temp = sprintf("%03u", $temp);
    $newfirst = `date +%Y%m%d`;
    $newfirst=~s/\n//;
    $myID = "LE".$newfirst."_".$temp;
    return $myID;
}

#Send files to the list of @hosts.
sub sendFiles{
    my ($host,$file,$rc,$filename,$numparts,@parts);
    @files = glob("$basedir/*");
    foreach $host (@hosts){
	foreach $file (@files){
	    if(-d $file){ next; }
	    $totalTransactions++;
	    print "Sending $file to $host...\n";
	    $rc = system("$gsi $file srour\@$host:/u/s/r/srour/vmr-gib/Incoming/");
#	    $rc = system("$gsi -E $host $dir $file");
	    @parts=split(/\//,$file);
            $numparts=@parts;
            $filename=$parts[$numparts-1];
	    if($rc != 0) {
		$rc = $rc/256;
		print "$gsi returned $rc. Couldnt send $filename to $host.\nQueuing for resend.\n";
		`cp $file $basedir/q/$filename\n`;
		push(@{ $HostQueues{$host} }, "$basedir/q/$filename");
		$numErrors++;
	    }
	    else {
		$numSuccess++;
	    }
	}
    }
    @qedfiles = glob("$basedir/*");
    foreach(@qedfiles){
	if(-f $_){
	    my $count = unlink $_;
	    if($count != 1){
		print "Error removing $_: $!\n";
	    }
	}
    }

}

#Send files in $basedir/q directory to hosts listed in %HostQueues
sub sendQueued{
    my ($file,$yes,$host,$rc);

    foreach $host (keys %HostQueues){
	foreach $file (@{$HostQueues{$host}}){
	    $totalTransactions++;
	    print "Should I attempt to send queued file $file to $host? (y/n) ";
	    $yes = <STDIN>;
	    if($yes =~ /y/i){
		print "Sending $file to $host\n";
		$rc = system("$gsi $file srour\@$host:/u/s/r/srour/vmr-gib/Incoming/");
#		$rc = system("$gsi -E $host $dir $file");
		@parts=split(/\//,$file);
		$numparts=@parts;
		$filename=$parts[$numparts-1];
		if($rc != 0){
		    $rc = $rc/256;
		    print "$gsi returned $rc. Couldnt send $filename to $host.\nQueuing for resend.\n";
		    $numErrors++;
		}
		else {
		    my $count = unlink $file;
		     if($count != 1){
			print "Error removing $file, $!\n";
		     }
		     else{
		    	$numSuccess++;
		     }
		}
	    }
	    else{
		$numIgnored++;
	    }
	}
    }
}

#Print some stats that we kept track of.
sub printStats{
    print "***** Session Statistics *****\n";
    print "$totalTransactions total transactions, $numSuccess sent, $numIgnored ignored, $numErrors errors (queued)\n";
}


1;








